@@ -4,18 +4,19 @@
 # Id: $Id$
 
 """
-Linear Solvers - KSP
+Linear Solvers (KSP)
 ====================
 
 The Scalable Linear Equations Solvers (KSP) component provides an
 easy-to-use interface to the combination of a Krylov subspace
 iterative method and a preconditioner (in the KSP and PC components,
-respectively) or a sequential direct solver. KSP users can set various
-Krylov subspace options at runtime via the options database. KSP users
-can also set KSP options directly in application by directly calling
-the KSP routines (e.g., 'KSP.setType()'). KSP components can be used
-directly to create and destroy solvers; this is not needed for users
-but is intended for library developers.
+respectively) or a sequential direct solver.
+
+KSP users can set various Krylov subspace options at runtime via the
+options database. KSP options can also be set programmatically by
+directly calling specific KSP routines (e.g., 'KSP.setType()'). KSP
+components can be used directly to create and destroy solvers; this is
+normally not needed for users but is intended for library developers.
 """
 
 # --------------------------------------------------------------------
@@ -24,7 +25,7 @@ __date__     = '$Date$'
 __version__  = '$Version$'
 __revision__ = '$Revision$'
 
-__docformat__ = 'reST'
+__docformat__ = 'reStructuredText'
 
 # --------------------------------------------------------------------
 
@@ -36,6 +37,8 @@ __all__ = ['KSP',
            'PCNone',
            'PCLU',
            'PCILU',
+           'PCCHOLESKY',
+           'PCICC',
            'PCShell',
            ]
 
@@ -57,7 +60,7 @@ class KSP(Object):
     
     class Type(object):
         """
-        KSP Types
+        KSP types.
         """
         RICHARDSON = _petsc.KSPRICHARDSON
         CHEBYCHEV  = _petsc.KSPCHEBYCHEV
@@ -120,8 +123,8 @@ class KSP(Object):
         Reason a Krylov method was said to have converged or diverged.
 
         + CONVERGED_ITERATING - this only occurs if
-        `KSP.getConvergedReason()` is called during `KSP.solve()`
-
+         `KSP.getConvergedReason()` is called during `KSP.solve()`
+         
         """
         # converged
         CONVERGED_RTOL             = _petsc.KSP_CONVERGED_RTOL
@@ -252,6 +255,12 @@ class KSP(Object):
         """
         _petsc.KSPSetNullSpace(self, nullspace)
 
+    def getNormType(self):
+        """
+        Get the norm that is used for convergence testing.
+        """
+        _petsc.KSPGetNormType(self)
+
     def setNormType(self, norm_type):
         """
         Set the norm that is used for convergence testing.
@@ -278,11 +287,11 @@ class KSP(Object):
         tolerances used by the default KSP convergence testers.
 
         :Parameters:
-           - `rtol` : the relative convergence tolerance (relative
-              decrease in the residual norm)
-           - `atol` : the absolute convergence tolerance (absolute
+           - `rtol`: the relative convergence tolerance (relative
+             decrease in the residual norm)
+           - `atol`: the absolute convergence tolerance (absolute
              size of the residual norm)
-           - `divtol` : the divergence tolerance (amount residual can
+           - `divtol`: the divergence tolerance (amount residual can
              increase before KSPDefaultConverged() concludes that the
              method is diverging)
            - `max_it`: maximum number of iterations to use
@@ -342,7 +351,7 @@ class KSP(Object):
         """
         Computes the explicit preconditioned operator.
 
-        .. notes:: This computation is done by applying the operators
+        .. note:: This computation is done by applying the operators
            to columns of the identity matrix.  Currently, this routine
            uses a dense matrix format when one processor is used and a
            sparse format otherwise.  This routine is costly in
@@ -370,6 +379,14 @@ class KSP(Object):
         """
         return _petsc.KSPGetIterationNumber(self)
 
+    def getConverged(self):
+        """
+        Get the reason the KSP iteration was stopped.
+        """
+        reason = _petsc.KSPGetConvergedReason(self)
+        reason_str = _petsc.KSPConvergedReasonString(reason)
+        return (reason, reason_str)
+
     def getConvergedReason(self):
         """
         Get the reason the KSP iteration was stopped.
@@ -379,12 +396,6 @@ class KSP(Object):
         """
         return _petsc.KSPGetConvergedReason(self)
 
-    def getConvergedReasonString(self):
-        """
-        """
-        reason = _petsc.KSPGetConvergedReason(self)
-        return _petsc.KSPConvergedReasonString(reason)
-
     def setConvergenceTest(self, convtest):
         """
         Set the function to be used to determine convergence.
@@ -398,25 +409,32 @@ class KSP(Object):
         """
         return _petsc.KSPGetResidualNorm(self)
 
-    def logResidualHistory(self, n=None, reset=True):
+    def setResidualHistory(self, length=None, reset=True):
         """
+        Log residual history.
         """
-        if n is None:
-            n = _petsc.PETSC_DECIDE
-        return _petsc.KSPLogResidualHistory(self, n, reset)
+        if length is None:
+            length = _petsc.PETSC_DECIDE
+        _petsc.KSPAllocResHistory(self, length, reset)
 
     def getResidualHistory(self):
         """
+        Get logged residual history.
         """
         return _petsc.KSPGetResidualHistory(self)
 
     def setComputeEigenvalues(self, flag):
         """
+        Set a flag so that the extreme eigenvalues values will be
+        calculated via a Lanczos or Arnoldi process as the linear
+        system is solved.
         """
         _petsc.KSPSetComputeEigenvalues(self, flag)
 
     def computeEigenvalues(self, neig=None):
         """
+        Compute the extreme eigenvalues for the preconditioned
+        operator.
         """
         maxit = _petsc.KSPGetTolerances(self)[3]
         if neig is None or neig > maxit:
@@ -431,6 +449,19 @@ class KSP(Object):
 
     def computeEigenvaluesExplicitly(self, neig=None):
         """
+        Compute all of the eigenvalues of the preconditioned operator
+        using LAPACK.
+
+        .. note:: This approach is very slow but will generally
+           provide accurate eigenvalue estimates.  This routine
+           explicitly forms a dense matrix representing the
+           preconditioned operator, and thus will run only for
+           relatively small problems, say n < 500.
+
+        .. tip:: Many users may just want to use the monitoring
+           routine KSPSingularValueMonitor() (which can be set with
+           option -ksp_singmonitor) to print the singular values at
+           each iteration of the linear solve.
         """
         A, _, _ = _petsc.KSPGetOperators(self)
         M, _ = _petsc.MatGetSize(A)
@@ -446,11 +477,16 @@ class KSP(Object):
 
     def setComputeSingularValues(self, flag):
         """
+        Set a flag so that the extreme singular values will be
+        calculated via a Lanczos or Arnoldi process as the linear
+        system is solved.
         """
         _petsc.KSPSetComputeSingularValues(self, flag)
 
     def computeExtremeSingularValues(self):
         """
+        Compute the extreme singular values for the preconditioned
+        operator
         """
         return tuple(_petsc.KSPComputeExtremeSingularValues(self))
 
@@ -481,12 +517,12 @@ class KSP(Object):
         """
         _petsc.KSPSolveTranspose(self, b, x)
     
-    pc     = property(getPC, setPC)
-    pcside = property(getPCSide, setPCSide)
+    pc      = property(getPC, setPC)
+    pc_side = property(getPCSide, setPCSide)
 
     nullspace = property(getNullSpace, setNullSpace)
 
-    norm_type = property(None, setNormType,
+    norm_type = property(getNormType, setNormType,
                          doc='norm type for convergence testing')
 
     rhs      = property(getRhs)
@@ -507,10 +543,9 @@ class KSP(Object):
     esv = property(computeExtremeSingularValues,
                    setComputeSingularValues)
 
-    iternum   = property(getIterationNumber)
-    resnorm   = property(getResidualNorm)
-    converged = property(getConvergedReason)
-    convstr   = property(getConvergedReasonString)
+    iternum    = its   = property(getIterationNumber)
+    resnorm    = rnorm = property(getResidualNorm)
+    converged  = property(getConverged)
 
 
 # --------------------------------------------------------------------
@@ -547,7 +582,7 @@ class PC(Object):
 
     class Type(object):
         """
-        PC Types
+        PC types.
         """
         NONE       = _petsc.PCNONE
         JACOBI     = _petsc.PCJACOBI
@@ -637,7 +672,7 @@ class PC(Object):
         Creates a preconditioner context.. The type can then be set with
         `setType()` (or perhaps `setFromOptions()`).
 
-        .. notes:: The default preconditioner on one processor is ILU
+        .. note:: The default preconditioner on one processor is ILU
            with 0 fill, on more than one it is BJACOBI with ILU() on
            each processor..
         """
@@ -649,7 +684,7 @@ class PC(Object):
 
         :Parameters:
           - `context`: user-defined context object implementig the
-             preconditioner interface.
+            preconditioner interface.
           - `comm`: MPI communicator (defaults to `COMM_WORLD`).
         """
         return _petsc.PCCreateShell(comm, context, self)
@@ -691,16 +726,22 @@ class PC(Object):
 
     def setDiagonalScale(self, vec):
         """
+        Indicates the left scaling to use to apply an additional left
+        and right scaling as needed by certain time-stepping codes.
         """
         _petsc.PCDiagonalScaleSet(self, vec)
 
     def diagonalScaleLeft(self, vec_in, vec_out):
         """
+        Indicates the left scaling to use to apply an additional left
+        and right scaling as needed by certain time-stepping codes.
         """
         _petsc.PCDiagonalScaleLeft(self, vec_in, vec_out)
 
     def diagonalScaleRight(self, vec_in, vec_out):
         """
+        Scales a vector by the right scaling as needed by certain
+        time-stepping codes.
         """
         _petsc.PCDiagonalScaleRight(self, vec_in, vec_out)
         
