@@ -18,7 +18,6 @@ typedef struct {
 static PetscErrorCode TSStep_User_Nonlinear(TS ts,PetscInt *steps,PetscReal *ptime)
 {
   PetscInt       i,its,lits;
-  Vec            sol = ts->vec_sol;
   TS_User        *tsuser = (TS_User*)ts->data;
   PetscErrorCode ierr;
   
@@ -30,19 +29,19 @@ static PetscErrorCode TSStep_User_Nonlinear(TS ts,PetscInt *steps,PetscReal *pti
     ts->nonlinear_its = 0;
   }
   *steps = -ts->steps;
-  ierr = TSMonitor(ts,ts->steps,ts->ptime,sol);CHKERRQ(ierr);
+  ierr = TSMonitor(ts,ts->steps,ts->ptime,ts->vec_sol);CHKERRQ(ierr);
   for (i=0; i<ts->max_steps; i++) {
     ierr = (*ts->ops->update)(ts, ts->ptime, &ts->time_step);CHKERRQ(ierr);
     if ((ts->ptime + ts->time_step) > ts->max_time) break;
     ts->ptime += ts->time_step;
-    ierr = VecCopy(sol,tsuser->update);CHKERRQ(ierr);
+    ierr = VecCopy(ts->vec_sol,tsuser->update);CHKERRQ(ierr);
     ierr = SNESSolve(ts->snes,PETSC_NULL,tsuser->update);CHKERRQ(ierr);
-    ierr = SNESGetNumberLinearIterations(ts->snes,&lits);CHKERRQ(ierr);
     ierr = SNESGetIterationNumber(ts->snes,&its);CHKERRQ(ierr);
+    ierr = SNESGetNumberLinearIterations(ts->snes,&lits);CHKERRQ(ierr);
     ts->nonlinear_its += its; ts->linear_its += lits;
-    ierr = VecCopy(tsuser->update,sol);CHKERRQ(ierr);
     ts->steps++;
-    ierr = TSMonitor(ts,ts->steps,ts->ptime,sol);CHKERRQ(ierr);
+    ierr = TSMonitor(ts,ts->steps,ts->ptime,tsuser->update);CHKERRQ(ierr);
+    ierr = VecCopy(tsuser->update,ts->vec_sol);CHKERRQ(ierr);
   }
   *steps += ts->steps;
   *ptime  = ts->ptime;
@@ -98,11 +97,20 @@ static PetscErrorCode TSUserJacobian(SNES snes,Vec x,Mat *AA,Mat *BB,MatStructur
 static PetscErrorCode TSSetUp_User_Nonlinear(TS ts)
 {
   TS_User        *tsuser = (TS_User*)ts->data;
+  Vec            func = PETSC_NULL;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = VecDuplicate(ts->vec_sol,&tsuser->update);CHKERRQ(ierr);  
-  ierr = VecDuplicate(ts->vec_sol,&tsuser->func);CHKERRQ(ierr);
+  ierr = PetscObjectQuery((PetscObject)ts,"__rhs_funcvec__",(PetscObject *)&func);CHKERRQ(ierr);
+  if (func != PETSC_NULL) {
+    ierr = PetscObjectReference((PetscObject)func);CHKERRQ(ierr);
+    tsuser->func = func;
+  } else {
+    ierr = VecDuplicate(ts->vec_sol,&tsuser->func);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(ts,tsuser->func);CHKERRQ(ierr);
+  }
+  ierr = VecDuplicate(ts->vec_sol,&tsuser->update);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(ts,tsuser->update);CHKERRQ(ierr);
   ierr = SNESSetFunction(ts->snes,tsuser->func,TSUserFunction,ts);CHKERRQ(ierr);
   ierr = SNESSetJacobian(ts->snes,ts->A,ts->B,TSUserJacobian,ts);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -173,7 +181,6 @@ PetscErrorCode PETSCTS_DLLEXPORT TSCreate_User(TS ts)
 
   ierr = SNESCreate(ts->comm,&ts->snes);CHKERRQ(ierr);
   ierr = PetscLogObjectParent(ts,ts->snes);CHKERRQ(ierr);
-
   
   PetscFunctionReturn(0);
 }
